@@ -14,7 +14,6 @@ use crate::options::*;
 use crate::util::OneWayOwned;
 use crate::writer::*;
 use std::borrow::Borrow;
-use std::io::Write;
 
 pub struct PlotElement
 {
@@ -226,7 +225,7 @@ impl PlotElement
 		}
 	}
 
-	fn write_args(&self, writer: &mut Writer, version: GnuplotVersion)
+	fn write_args(&self, writer: &mut dyn Writer, version: GnuplotVersion)
 	{
 		let options = &self.options;
 		match self.source_type
@@ -433,7 +432,7 @@ impl PlotElement
 		}
 	}
 
-	fn write_data(&self, writer: &mut Writer)
+	fn write_data(&self, writer: &mut dyn Writer)
 	{
 		for d in &self.data
 		{
@@ -442,6 +441,59 @@ impl PlotElement
 	}
 }
 
+pub struct LabelData
+{
+	pub label_type: LabelType,
+	pub text: String,
+	pub options: Vec<LabelOption<String>>,
+}
+
+impl LabelData
+{
+	fn new(label_type: LabelType) -> Self
+	{
+		Self {
+			label_type: label_type,
+			text: "".into(),
+			options: vec![],
+		}
+	}
+
+	pub fn set(&mut self, text: String, options: Vec<LabelOption<String>>)
+	{
+		self.text = text;
+		self.options = options;
+	}
+
+	pub fn write_out_commands(&self, writer: &mut dyn Writer)
+	{
+		let w = writer;
+		w.write_str("set ");
+
+		self.label_type.write_label_str(w);
+
+		w.write_str(" \"");
+		w.write_str(&self.text);
+		w.write_str("\"");
+
+		write_out_label_options(self.label_type, &self.options[..], w);
+
+		w.write_str("\n");
+	}
+
+	pub fn reset_state(&self, writer: &mut dyn Writer)
+	{
+		match self.label_type
+		{
+			Label(tag, ..) =>
+			{
+				writeln!(writer, "unset label {}", tag);
+			}
+			_ => (),
+		}
+	}
+}
+
 #[derive(Copy, Clone)]
 pub enum LabelType
 {
@@ -450,7 +502,7 @@ pub enum LabelType
 	ZLabel,
 	CBLabel,
 	TitleLabel,
-	Label(Coordinate, Coordinate),
+	Label(i32, Coordinate, Coordinate),
 	AxesTicks,
 }
 
@@ -464,16 +516,59 @@ impl LabelType
 			_ => false,
 		}
 	}
+
+	fn write_label_str(&self, w: &mut dyn Writer)
+	{
+		match *self
+		{
+			XLabel =>
+			{
+				w.write_str("xlabel");
+			}
+			YLabel =>
+			{
+				w.write_str("ylabel");
+			}
+			ZLabel =>
+			{
+				w.write_str("zlabel");
+			}
+			CBLabel =>
+			{
+				w.write_str("cblabel");
+			}
+			TitleLabel =>
+			{
+				w.write_str("title");
+			}
+			Label(tag, ..) =>
+			{
+				write!(w, "label {}", tag);
+			}
+			_ => panic!("Invalid label type"),
+		}
+	}
+
+	fn from_axis(axis_type: TickAxis) -> Self
+	{
+		match axis_type
+		{
+			XTickAxis => XLabel,
+			YTickAxis => YLabel,
+			ZTickAxis => ZLabel,
+			CBTickAxis => CBLabel,
+		}
+	}
 }
 
 pub fn write_out_label_options(
-	label_type: LabelType, options: &[LabelOption<String>], writer: &mut Writer,
+	label_type: LabelType, options: &[LabelOption<String>], writer: &mut dyn Writer,
 )
 {
 	let w = writer;
 	match label_type
 	{
-		Label(x, y) =>
+		Label(_, x, y) =>
 		{
 			write!(w, " at {},{} front", x, y);
 		}
@@ -673,6 +768,7 @@ pub struct AxisData
 	pub mgrid: bool,
 	pub is_time: bool,
 	pub show: bool,
+	pub label: LabelData,
 	pub options: Vec<PlotOption<String>>,
 }
 
@@ -693,11 +789,12 @@ impl AxisData
 			mgrid: false,
 			is_time: false,
 			show: false,
+			label: LabelData::new(LabelType::from_axis(axis)),
 			options: vec![],
 		}
 	}
 
-	pub fn write_out_commands(&self, w: &mut Writer, version: GnuplotVersion)
+	pub fn write_out_commands(&self, w: &mut dyn Writer, version: GnuplotVersion)
 	{
 		if self.axis != TickAxis::CBTickAxis
 		{
@@ -929,6 +1026,8 @@ impl AxisData
 			}
 		}
 		w.write_str("\n");
+		self.label.write_out_commands(w);
+		w.write_str("\n");
 	}
 
 	pub fn set_ticks_custom<T: DataType, TL: IntoIterator<Item = Tick<T, String>>>(
@@ -1028,19 +1127,74 @@ enum DataSourceType
 	SizedArray(f64, f64, f64, f64),
 }
 
+pub struct Margins
+{
+	pub left: Option<f32>,
+	pub right: Option<f32>,
+	pub top: Option<f32>,
+	pub bottom: Option<f32>,
+}
+
+impl Margins
+{
+	pub fn new() -> Self
+	{
+		Margins {
+			left: None,
+			right: None,
+			top: None,
+			bottom: None,
+		}
+	}
+
+	pub fn write_out_commands(&self, w: &mut dyn Writer)
+	{
+		let mut write_margin = |margin, v| {
+			write!(w, "set {}", margin);
+			if let Some(v) = v
+			{
+				write!(w, " at screen {}", v);
+			}
+			w.write_str("\n");
+		};
+
+		write_margin("lmargin", self.left);
+		write_margin("rmargin", self.right);
+		write_margin("tmargin", self.top);
+		write_margin("bmargin", self.bottom);
+	}
+}
+
+#[derive(Copy, Clone)]
+pub struct Position
+{
+	x: f64,
+	y: f64,
+}
+
+#[derive(Copy, Clone)]
+pub struct Size
+{
+	w: f64,
+	h: f64,
+}
+
 pub struct AxesCommonData
 {
-	pub commands: Vec<u8>,
 	pub grid_options: Vec<PlotOption<String>>,
 	pub minor_grid_options: Vec<PlotOption<String>>,
 	pub grid_front: bool,
 	pub elems: Vec<PlotElement>,
-	pub grid_rows: u32,
-	pub grid_cols: u32,
-	pub grid_pos: Option<u32>,
 	pub x_axis: AxisData,
 	pub y_axis: AxisData,
 	pub cb_axis: AxisData,
+	pub labels: Vec<LabelData>,
+	pub title: LabelData,
+	pub position: Option<Position>,
+	pub size: Option<Size>,
+	pub aspect_ratio: AutoOption<f64>,
+	pub margins: Margins,
+	pub palette: PaletteType,
 }
 
 impl AxesCommonData
@@ -1048,21 +1202,24 @@ impl AxesCommonData
 	pub fn new() -> AxesCommonData
 	{
 		AxesCommonData {
-			commands: vec![],
 			grid_options: vec![],
 			minor_grid_options: vec![],
 			grid_front: false,
 			elems: Vec::new(),
-			grid_rows: 0,
-			grid_cols: 0,
-			grid_pos: None,
 			x_axis: AxisData::new(XTickAxis),
 			y_axis: AxisData::new(YTickAxis),
 			cb_axis: AxisData::new(CBTickAxis),
+			labels: vec![],
+			title: LabelData::new(TitleLabel),
+			position: None,
+			size: None,
+			aspect_ratio: Auto,
+			margins: Margins::new(),
+			palette: COLOR,
 		}
 	}
 
-	pub fn write_grid_options(&self, c: &mut Writer, axes: &[TickAxis], version: GnuplotVersion)
+	pub fn write_grid_options(&self, c: &mut dyn Writer, axes: &[TickAxis], version: GnuplotVersion)
 	{
 		if !axes.is_empty()
 		{
@@ -1098,7 +1255,7 @@ impl AxesCommonData
 	}
 
 	pub fn write_line_options(
-		c: &mut Writer, options: &[PlotOption<String>], version: GnuplotVersion,
+		c: &mut dyn Writer, options: &[PlotOption<String>], version: GnuplotVersion,
 	)
 	{
 		let mut found = false;
@@ -1136,7 +1293,7 @@ impl AxesCommonData
 	}
 
 	pub fn write_color_options(
-		c: &mut Writer, options: &[PlotOption<String>], default: Option<&str>,
+		c: &mut dyn Writer, options: &[PlotOption<String>], default: Option<&str>,
 	)
 	{
 		let mut col = default;
@@ -1156,15 +1313,107 @@ impl AxesCommonData
 		}
 	}
 
-	pub fn write_out_commands(&self, writer: &mut Writer, version: GnuplotVersion)
+	pub fn write_out_commands(
+		&self, writer: &mut dyn Writer, auto_layout: bool, version: GnuplotVersion,
+	)
 	{
-		writer.write_all(&self.commands[..]);
-		self.x_axis.write_out_commands(writer, version);
-		self.y_axis.write_out_commands(writer, version);
-		self.cb_axis.write_out_commands(writer, version);
+		let w = writer;
+		if let Some(pos) = self.position
+		{
+			writeln!(w, "set origin {:.12e},{:.12e}", pos.x, pos.y);
+		}
+		else if !auto_layout
+		{
+			writeln!(w, "set origin");
+		}
+		if let Some(size) = self.size
+		{
+			writeln!(w, "set size {:.12e},{:.12e}", size.w, size.h);
+		}
+		else if !auto_layout
+		{
+			writeln!(w, "set size");
+		}
+
+		match self.aspect_ratio
+		{
+			Fix(r) =>
+			{
+				writeln!(w, "set size ratio {:.12e}", r);
+			}
+			Auto =>
+			{
+				writeln!(w, "set size noratio");
+			}
+		}
+		self.margins.write_out_commands(w);
+
+		match self.palette
+		{
+			Gray(gamma) =>
+			{
+				assert!(gamma > 0.0, "Gamma must be positive");
+				writeln!(w, "set palette gray gamma {:.12e}", gamma);
+			}
+			Formula(r, g, b) =>
+			{
+				assert!(r >= -36 && r <= 36, "Invalid r formula!");
+				assert!(g >= -36 && g <= 36, "Invalid g formula!");
+				assert!(b >= -36 && b <= 36, "Invalid b formula!");
+				writeln!(w, "set palette rgbformulae {},{},{}", r, g, b);
+			}
+			CubeHelix(start, rev, sat, gamma) =>
+			{
+				assert!(sat >= 0.0, "Saturation must be non-negative");
+				assert!(gamma > 0.0, "Gamma must be positive");
+				writeln!(
+					w,
+					"set palette cubehelix start {:.12e} cycles {:.12e} saturation {:.12e} gamma {:.12e}",
+					start, rev, sat, gamma
+				);
+			}
+			Custom(ref entries) =>
+			{
+				if entries.is_empty()
+				{
+					panic!("Need at least 1 element in a custom palette");
+				}
+				write!(w, "set palette defined (");
+
+				let mut first = true;
+				let mut old_x = 0.0;
+				for &(x, r, g, b) in entries
+				{
+					if first
+					{
+						old_x = x;
+						first = false;
+					}
+					else
+					{
+						write!(w, ",");
+					}
+					assert!(x >= old_x, "The gray levels must be non-decreasing!");
+					old_x = x;
+
+					write!(w, "{:.12e} {:.12e} {:.12e} {:.12e}", x, r, g, b);
+				}
+
+				writeln!(w, ")");
+			}
+		}
+
+		self.x_axis.write_out_commands(w, version);
+		self.y_axis.write_out_commands(w, version);
+		self.cb_axis.write_out_commands(w, version);
+		self.title.write_out_commands(w);
+		for label in &self.labels
+		{
+			label.write_out_commands(w);
+		}
 	}
 
-	pub fn write_out_elements(&self, cmd: &str, writer: &mut Writer, version: GnuplotVersion)
+	pub fn write_out_elements(&self, cmd: &str, writer: &mut dyn Writer, version: GnuplotVersion)
 	{
 		write!(writer, "{}", cmd);
 
@@ -1187,33 +1436,12 @@ impl AxesCommonData
 		}
 	}
 
-	pub fn set_label_common(
-		&mut self, label_type: LabelType, text: &str, options: &[LabelOption<&str>],
-	)
+	pub fn reset_state(&self, writer: &mut dyn Writer)
 	{
-		let c = &mut self.commands;
-
-		c.write_str("set ");
-
-		let label_str = match label_type
+		for label in &self.labels
 		{
-			XLabel => "xlabel",
-			YLabel => "ylabel",
-			ZLabel => "zlabel",
-			CBLabel => "cblabe",
-			TitleLabel => "title",
-			Label(..) => "label",
-			_ => panic!("Invalid label type"),
-		};
-		c.write_str(label_str);
-
-		c.write_str(" \"");
-		c.write_str(text);
-		c.write_str("\"");
-
-		write_out_label_options(label_type, &options.to_one_way_owned()[..], &mut *c);
-
-		c.write_str("\n");
+			label.reset_state(writer);
+		}
 	}
 }
 
@@ -1237,12 +1465,16 @@ pub trait AxesCommon: AxesCommonPrivate
 		assert!(nrow > 0);
 		assert!(ncol > 0);
 		assert!(pos < nrow * ncol);
-		{
-			let c = self.get_common_data_mut();
-			c.grid_rows = nrow;
-			c.grid_cols = ncol;
-			c.grid_pos = Some(pos);
-		}
+		let width = 1.0 / (ncol as f64);
+		let height = 1.0 / (nrow as f64);
+		let x = (pos % ncol) as f64 * width;
+		let y = 1.0 - (1.0 + (pos / ncol) as f64) * height;
+
+		self.get_common_data_mut().position = Some(Position { x: x, y: y });
+		self.get_common_data_mut().size = Some(Size {
+			w: width,
+			h: height,
+		});
 		self
 	}
 
@@ -1253,13 +1485,7 @@ pub trait AxesCommon: AxesCommonPrivate
 	/// * `y` - Y position. Ranges from 0 to 1
 	fn set_pos<'l>(&'l mut self, x: f64, y: f64) -> &'l mut Self
 	{
-		self.get_common_data_mut().grid_pos = None;
-		writeln!(
-			&mut self.get_common_data_mut().commands,
-			"set origin {:.12e},{:.12e}",
-			x,
-			y
-		);
+		self.get_common_data_mut().position = Some(Position { x: x, y: y });
 		self
 	}
 
@@ -1269,12 +1495,7 @@ pub trait AxesCommon: AxesCommonPrivate
 	/// * `h` - Height. Ranges from 0 to 1
 	fn set_size<'l>(&'l mut self, w: f64, h: f64) -> &'l mut Self
 	{
-		writeln!(
-			&mut self.get_common_data_mut().commands,
-			"set size {:.12e},{:.12e}",
-			w,
-			h
-		);
+		self.get_common_data_mut().size = Some(Size { w: w, h: h });
 		self
 	}
 
@@ -1283,21 +1504,7 @@ pub trait AxesCommon: AxesCommonPrivate
 	/// * `ratio` - The aspect ratio. Set to Auto to return the ratio to default
 	fn set_aspect_ratio<'l>(&'l mut self, ratio: AutoOption<f64>) -> &'l mut Self
 	{
-		{
-			let c = &mut self.get_common_data_mut().commands as &mut Writer;
-
-			match ratio
-			{
-				Fix(r) =>
-				{
-					writeln!(c, "set size ratio {:.12e}", r);
-				}
-				Auto =>
-				{
-					writeln!(c, "set size noratio");
-				}
-			}
-		}
+		self.get_common_data_mut().aspect_ratio = ratio;
 		self
 	}
 
@@ -1313,7 +1520,9 @@ pub trait AxesCommon: AxesCommonPrivate
 	fn set_x_label<'l>(&'l mut self, text: &str, options: &[LabelOption<&str>]) -> &'l mut Self
 	{
 		self.get_common_data_mut()
-			.set_label_common(XLabel, text, options);
+			.x_axis
+			.label
+			.set(text.into(), options.to_one_way_owned());
 		self
 	}
 
@@ -1321,7 +1530,9 @@ pub trait AxesCommon: AxesCommonPrivate
 	fn set_y_label<'l>(&'l mut self, text: &str, options: &[LabelOption<&str>]) -> &'l mut Self
 	{
 		self.get_common_data_mut()
-			.set_label_common(YLabel, text, options);
+			.y_axis
+			.label
+			.set(text.into(), options.to_one_way_owned());
 		self
 	}
 
@@ -1329,7 +1540,9 @@ pub trait AxesCommon: AxesCommonPrivate
 	fn set_cb_label<'l>(&'l mut self, text: &str, options: &[LabelOption<&str>]) -> &'l mut Self
 	{
 		self.get_common_data_mut()
-			.set_label_common(CBLabel, text, options);
+			.cb_axis
+			.label
+			.set(text.into(), options.to_one_way_owned());
 		self
 	}
 
@@ -1345,7 +1558,8 @@ pub trait AxesCommon: AxesCommonPrivate
 	fn set_title<'l>(&'l mut self, text: &str, options: &[LabelOption<&str>]) -> &'l mut Self
 	{
 		self.get_common_data_mut()
-			.set_label_common(TitleLabel, text, options);
+			.title
+			.set(text.into(), options.to_one_way_owned());
 		self
 	}
 
@@ -1367,8 +1581,12 @@ pub trait AxesCommon: AxesCommonPrivate
 		&'l mut self, text: &str, x: Coordinate, y: Coordinate, options: &[LabelOption<&str>],
 	) -> &'l mut Self
 	{
-		self.get_common_data_mut()
-			.set_label_common(Label(x, y), text, options);
+		{
+			let labels = &mut self.get_common_data_mut().labels;
+			let mut label = LabelData::new(Label(labels.len() as i32 + 1, x, y));
+			label.set(text.into(), options.to_one_way_owned());
+			labels.push(label);
+		}
 		self
 	}
 
@@ -1703,20 +1921,21 @@ pub trait AxesCommon: AxesCommonPrivate
 	///
 	/// # Arguments
 	///
-	/// * `margins` - The values of margins to be overrided. Specified as a fraction of the
+	/// * `margins` - The values of margins to be overriden. Specified as a fraction of the
 	///               full drawing area, ranging from 0 to 1
 	fn set_margins<'l>(&'l mut self, margins: &[MarginSide]) -> &'l mut Self
 	{
 		{
-			let c = &mut self.get_common_data_mut().commands as &mut Writer;
+			let m = &mut self.get_common_data_mut().margins;
+			*m = Margins::new();
 			for &s in margins.iter()
 			{
 				match s
 				{
-					MarginTop(frac) => writeln!(c, "set tmargin at screen {}", frac),
-					MarginBottom(frac) => writeln!(c, "set bmargin at screen {}", frac),
-					MarginLeft(frac) => writeln!(c, "set lmargin at screen {}", frac),
-					MarginRight(frac) => writeln!(c, "set rmargin at screen {}", frac),
+					MarginLeft(frac) => m.left = Some(frac),
+					MarginRight(frac) => m.right = Some(frac),
+					MarginTop(frac) => m.top = Some(frac),
+					MarginBottom(frac) => m.bottom = Some(frac),
 				};
 			}
 		}
@@ -1729,79 +1948,7 @@ pub trait AxesCommon: AxesCommonPrivate
 	/// * `palette` - What palette type to use
 	fn set_palette(&mut self, palette: PaletteType) -> &mut Self
 	{
-		{
-			let c = &mut self.get_common_data_mut().commands as &mut Writer;
-			match palette
-			{
-				Gray(gamma) =>
-				{
-					assert!(gamma > 0.0, "Gamma must be positive");
-					writeln!(c, "set palette gray gamma {:.12e}", gamma);
-				}
-				Formula(r, g, b) =>
-				{
-					assert!(r >= -36 && r <= 36, "Invalid r formula!");
-					assert!(g >= -36 && g <= 36, "Invalid g formula!");
-					assert!(b >= -36 && b <= 36, "Invalid b formula!");
-					writeln!(c, "set palette rgbformulae {},{},{}", r, g, b);
-				}
-				CubeHelix(start, rev, sat, gamma) =>
-				{
-					assert!(sat >= 0.0, "Saturation must be non-negative");
-					assert!(gamma > 0.0, "Gamma must be positive");
-					writeln!(
-						c,
-						"set palette cubehelix start {:.12e} cycles {:.12e} saturation {:.12e} gamma {:.12e}",
-						start, rev, sat, gamma
-					);
-				}
-			}
-		}
-		self
-	}
-
-	/// Sets a custom palette used for 3D surface and image plots. A custom palette
-	/// is specified by a sequence of 4-tuples (with at least one element). The first
-	/// element is the grayscale value that is mapped to the remaining three elements
-	/// which specify the red, green and blue components of the color.
-	/// The grayscale values must be non-decreasing. All values must range from 0 to 1.
-	///
-	/// # Arguments
-	/// * `palette_generator` - The palette generator
-	fn set_custom_palette<T: IntoIterator<Item = (f32, f32, f32, f32)>>(
-		&mut self, palette_generator: T,
-	) -> &mut Self
-	{
-		{
-			let c = &mut self.get_common_data_mut().commands as &mut Writer;
-			write!(c, "set palette defined (");
-
-			let mut first = true;
-			let mut old_x = 0.0;
-			for (x, r, g, b) in palette_generator
-			{
-				if first
-				{
-					old_x = x;
-					first = false;
-				}
-				else
-				{
-					write!(c, ",");
-				}
-				assert!(x >= old_x, "The gray levels must be non-decreasing!");
-				old_x = x;
-
-				write!(c, "{:.12e} {:.12e} {:.12e} {:.12e}", x, r, g, b);
-			}
-
-			if first
-			{
-				panic!("Need at least 1 element in the generator");
-			}
-
-			writeln!(c, ")");
-		}
+		self.get_common_data_mut().palette = palette;
 		self
 	}
 }
