@@ -2,7 +2,6 @@ use std::error;
 use std::fmt;
 use std::io;
 
-#[derive(Debug)]
 pub struct GnuplotInitError
 {
 	inner: Box<dyn error::Error + 'static>,
@@ -24,8 +23,24 @@ impl fmt::Display for GnuplotInitError
 	{
 		write!(
 			f,
-			"Couldn't spawn gnuplot. Make sure it is installed and available in PATH.\n{}",
+			"Couldn't spawn gnuplot. Make sure it is installed and available in PATH.\nCause: {}",
 			self.inner
 		)
 	}
 }
+
+impl fmt::Debug for GnuplotInitError
+{
+	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result
+	{
+		write!(f, "{}", self)
+	}
+}
+
+impl error::Error for GnuplotInitError
+{
+	fn source(&self) -> Option<&(dyn error::Error + 'static)>
+	{
+		Some(&*self.inner)
+	}
+}
